<?php
####################################################
#                                                  #
# █████   ████   ████  ██████  ████   ████  ████   #
# ██  ██   ██   ██  ██ █ ██ █ ██  ██ ██  ██  ██    #
# ██  ██   ██   ██       ██   ██  ██ ██  ██  ██    #
# █████    ██   ██       ██   ██  ██ ██  ██  ██    #
# ██       ██   ██       ██   ██  ██ ██  ██  ██    #
# ██       ██   ██  ██   ██   ██  ██ ██  ██  ██ ██ #
# ██      ████   ████   ████   ████   ████  ██████ #
#                                                  #
####################################################

/**
 ****************************************
 * Description
 * @name        : pictool_class.php
 * @Package     : 
 * @Author      : Bronco
 * @Description : A simple class to create, load, manipulate images with GD in PHP
 * @Version     : 1.2 (added mirror+flip)
 ****************************************
 **/
/* Example:
$pic=new pictool;
$pic->load('url');
$pic->rotate(90);
$pic->display();
*/
class pictool{
	private $image=null; # string
	private $rendered=null; # string
	private $format='jpg'; # string
	private $quality=90;#integer
	private $w; # integer
	private $h; # integer
	private $colors; # array
	private $transparent_color; # string
	private $font_path; # string
	private $font_size=16; # integer
	//private $font_color; # string

	public function __construct($array=[]){
		foreach ($array as $key => $value) {
			if (property_exists($this, $key)){
				$this->$key=$value;
			}
		}
		if ($this->w&&$this->h){
			$this->new($this->w,$this->h);
		}
	}

	###########################
	# METHODS #################
	###########################

	# private
	private function file_curl_contents($url){
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,  FALSE);     
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);     
		curl_setopt($ch, CURLOPT_URL, $url);     
		if (!ini_get("safe_mode") && !ini_get('open_basedir') ) {curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);}    
		curl_setopt($ch, CURLOPT_MAXREDIRS, 10); 
		curl_setopt($ch, CURLOPT_REFERER, 'http://oudanstoncul.free.fr');// notez le referer "custom"
		$data = curl_exec($ch);     
		curl_close($ch);     
		return $data; 
	}
	private function separatRGB($color){
	    $color=$this->normalizeColor($color);
	    if (strlen($color)==3){
	        $color=$color[0].$color[0].$color[1].$color[1].$color[2].$color[2];
	    }
	    $RGB=array();
	    $RGB['r']=hexdec(substr($color, 0,2));
	    $RGB['g']=hexdec(substr($color, 2,2));
	    $RGB['b']=hexdec(substr($color, 4,2));  
	    return $RGB;
	}
	private function normalizeColor($str){
		$color=preg_replace('#[^0-9A-Fa-f]#','',$str);
		$nb=strlen($color);
	    if ($nb==3){
	        $color=$color[0].$color[0].$color[1].$color[1].$color[2].$color[2];
	    }elseif($nb>6){
	    	$color=substr($color, 0,6);
	    }elseif($nb<3){
	    	$color.=str_repeat('0', 3-$nb);
	    }
	    return $color;
	}

	private function percentToSize($percent,$size){
		/* returns INT
			* if $percent is not string => $percent (like a correct size)
			* if $percent is string but without '%' => convert string to int
			* if $percent is string and has '%' => calculated new size
		*/
		if (is_string($percent)){
			$p=intval($percent);
			if (strpos($percent, '%')!==false){
				return round(($size*$p)/100);
			}else{
				return $p;
			}
			
		}else{
			return $percent;
		}
	}

	private function calculateTextBox($text,$fontFile,$fontSize,$fontAngle) {
	    /************ Not mine
	    simple function that calculates the *exact* bounding box (single pixel precision).
	    The function returns an associative array with these keys:
	    left, top:  coordinates you will pass to imagettftext
	    width, height: dimension of the image you have to create
	    *************/
	    $rect = imagettfbbox($fontSize,$fontAngle,$fontFile,$text);
	    $minX = min(array($rect[0],$rect[2],$rect[4],$rect[6]));
	    $maxX = max(array($rect[0],$rect[2],$rect[4],$rect[6]));
	    $minY = min(array($rect[1],$rect[3],$rect[5],$rect[7]));
	    $maxY = max(array($rect[1],$rect[3],$rect[5],$rect[7]));
	   
	    return array(
	     "left"   => abs($minX) - 1,
	     "top"    => abs($minY) - 1,
	     "width"  => $maxX - $minX,
	     "height" => $maxY - $minY,
	     "box"    => $rect
	    );
	} 

	# create a new color for the current image
	# and adds it to the colors attribute array
	# the $name is used later to call this color (like fill("backcolor"))
	public function newColor($name,$color){
		if (!$name||!$color||!is_string($name)||!is_string($color)){return false;}
		
		$color=$this->separatRGB($color);
		$this->colors[$name]=imagecolorallocate($this->getImage(), $color['r'], $color['g'], $color['b']);
		return $this->colors[$name];
	}

	# make a color transparent
	# $color = HTML color or named color (see newColor())
	public function makeTransparent($color='#000'){
		if (!$color||!is_string($color)){return false;}
		$color=$this->getColor($color);
		$i=$this->getImage();
		imagecolortransparent($i, $color);
		imagealphablending($i, false);
    	imagesavealpha($i, true);
	}

	# create a new image
	###########################
	public function new($w,$h){
		$this->setImage(imagecreatetruecolor($w,$h));
		return $this->image;
	}

	# create an image loading 
	# it form a URL
	###########################
	public function load($url){
		if ($downloaded_pic=$this->file_curl_contents($url)){
			if (!$pic=imagecreatefromstring($downloaded_pic)){
				exit('Error in picture format');
			}
			$this->setImage($pic);
			list($w,$h) = getimagesizefromstring($downloaded_pic);
			$this->setW($w);
			$this->setH($h);
		}else{
			return false;
		}
	}

	####################################################
	#                                                  #
	# ▒▒▒▒▒▒  ▒▒▒▒  ▒▒▒▒   ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒   ▒▒▒▒  #
	# ▒▒       ▒▒    ▒▒    ▒ ▒▒ ▒ ▒▒     ▒▒  ▒▒ ▒▒   ▒ #
	# ▒▒▒▒▒    ▒▒    ▒▒      ▒▒   ▒▒▒▒▒  ▒▒▒▒▒    ▒▒   #
	# ▒▒       ▒▒    ▒▒ ▒▒   ▒▒   ▒▒     ▒▒  ▒▒ ▒   ▒▒ #
	# ▒▒      ▒▒▒▒  ▒▒▒▒▒▒  ▒▒▒▒  ▒▒▒▒▒▒ ▒▒  ▒▒  ▒▒▒▒  #
	#                                                  #
	####################################################
	# apply a filter
	# $filters as array ['filtername'=>value]
	###########################
	public function filters($filters=[]){
		$i=$this->getImage();
		if (is_array($filters)){
			foreach($filters as $filter=>$amount){
				$constant='IMG_FILTER_'.strtoupper($filter);
				if (is_array($amount)){
					imagefilter($i,constant($constant), ...$amount);
				}else{
					imagefilter($i,constant($constant), $amount);
				}
			}
		}elseif (is_string($filters)){
			$constant='IMG_FILTER_'.strtoupper($filters);
			imagefilter($i,constant($constant),1);
		}
		
	}
	# aliases
	# You can create your own aliases (like sepia())
	public function emboss(){					$this->filters('emboss');}
	public function edges(){					$this->filters('edgedetect');}
	public function negative(){					$this->filters('negate');}
	public function sketch(){					$this->filters('mean_removal');}
	public function grey(){						$this->filters('grayscale');}
	public function sepia(){					$this->filters('grayscale');$this->colorize([100,50,0]);}
	public function gaussian(){					$this->filters('gaussian_blur');}
	public function blur(){						$this->filters('selective_blur');}// ?!
	public function smooth($amount=0){			$this->filters(['smooth'=>$amount]);} // ?!
	public function pixel($size=5,$blur=1){ 	$this->filters(['pixelate'=>[$size,$blur]]);}
	public function scatter($x=5,$y=5){ 		$this->filters(['scatter'=>[$x,$y]]);} // ?!
	public function brightness($amount=0){		$this->filters(['brightness'=>$amount]);}
	public function contrast($amount=0){		$this->filters(['contrast'=>-$amount]);}
	public function colorize($color,$alpha=0){		
		if (is_string($color)){
			$color=$this->separatRGB($color);
		}
		$color['alpha']=$alpha;
		$this->filters(['colorize'=>array_values($color)]);
	}

	##############################################
	#                                            #
	# ▒     ▒  ▒▒▒▒  ▒▒▒▒▒   ▒▒▒▒  ▒▒▒▒▒▒ ▒▒  ▒▒ #
	# ▒▒   ▒▒ ▒▒  ▒▒ ▒▒  ▒▒   ▒▒   ▒▒     ▒▒  ▒▒ #
	# ▒▒▒ ▒▒▒ ▒▒  ▒▒ ▒▒  ▒▒   ▒▒   ▒▒▒▒▒   ▒▒▒▒  #
	# ▒▒ ▒ ▒▒ ▒▒  ▒▒ ▒▒  ▒▒   ▒▒   ▒▒       ▒▒   #
	# ▒▒   ▒▒  ▒▒▒▒  ▒▒▒▒▒   ▒▒▒▒  ▒▒       ▒▒   #
	#                                            #
	##############################################
	# merge the current image with the given one ($image)
	public function merge($image,$x=0,$y=0){
		if (get_class($image)=='pictool'){
			# with this, you can merge the current image with a pictool object
			$image=$image->getImage();
		}
		imagecopymerge($this->getImage(),$image,$x,$y,0,0,imagesx($image),imagesy($image),100);
	}

	# crop the current image (dimensions can be px or %)
	public function crop($x=0,$y=0,$w,$h){
		$i=$this->getImage();
		$ix=imagesx($i);
		$iy=imagesy($i);
		# converts $x,$y,$w & $h if needed
		$w=$this->percentToSize($w,$ix);
		$h=$this->percentToSize($h,$iy);
		$x=$this->percentToSize($x,$ix);
		$y=$this->percentToSize($y,$iy);
		$this->setImage(imagecrop($i,['x'=>$x,'y'=>$y,'width'=>$w,'height'=>$h]));
	}

	# resize the current image 
	# dimensions can be px or %, 
	# put -1 to keep aspect ratio, ex: resize(256,-1)
	public function resize($w=-1,$h=-1){
		if (!$w&&!$h){return;}
		$i=$this->getImage();
		$ix=imagesx($i);
		$iy=imagesy($i);

		# converts $w & $h if needed
		$w=$this->percentToSize($w,$ix);
		$h=$this->percentToSize($h,$iy);
		
		if (!$h&&$w){
			$h=($w*$iy)/$ix;
		}elseif($h&&!$w){
			$w=($h*$ix)/$iy;
		}
		$this->setImage(imagescale($i,$w,$h));
	}


	# mirror the current image 
	public function mirror(){
		$this->flip('horizontal');
	}

	# flip the current image 
	# $direction: 0 = horizontal, 1= vertical, 2= both
	public function flip($direction=0){
		$i=$this->getImage();
		if ($direction==0||$direction=='0'||$direction=='horizontal'||$direction=='h'){
			imageflip($i,IMG_FLIP_HORIZONTAL);
		}
		if ($direction==1||$direction=='1'||$direction=='vertical'||$direction=='v'){
			imageflip($i,IMG_FLIP_VERTICAL);
		}
		if ($direction==2||$direction=='2'||$direction=='both'||$direction=='b'){
			imageflip($i,IMG_FLIP_BOTH);
		}
			
		$this->setImage($i);
	}



	# rotate the current image 
	public function rotate($angle=0,$color='#000'){
		if (is_string($angle)){$angle=intval($angle);}
		$this->setImage(imagerotate($this->getImage(), $angle, $this->getColor($color)));
	}

	# fill with a color in FFFFFF format
	# or the color name used in newColor() function
	public function fill($color){
		imagefill($this->getImage(),0,0,$this->getColor($color));
	}

	# add text into the current image
	# example :
	/* 	$picture->setFont('./old_stamper.ttf',32);
		$picture->text("so cool!",'left','top','#f00',0);
		$picture->text("so cool!",'center','center','#000',45);
		$picture->text("so cool!",'center','center','#0f0',45);
		$picture->text("so cool!",'right','bottom','#00f',0);
	*/
	public function text($text='',$x=0,$y=0,$color='#000',$size=null,$font=null,$angle=0){

		$this->font_size=$size?$size:$this->font_size;
		$this->font_path=$font?$font:$this->font_path;

		if (!$this->font_path||!is_file($this->font_path)){
			trigger_error('Font file not found');
			return false;
		}
		$i=$this->getImage();
		$ix=imagesx($i);
		$iy=imagesy($i);

		# converts $w & $h if needed
		$x=$this->percentToSize($x,$ix);
		$h=$this->percentToSize($y,$iy);

		$isstrX=is_string($x);
		$isstrY=is_string($y);

		if ($isstrY||$isstrX){
			$textbox=$this->calculateTextBox($text,$this->font_path,$this->font_size,$angle);
			if ($isstrY){
				if ($y=='top'){
					$y=$textbox['top'];
				}
				elseif ($y=='center'){
					$y=($this->h-$textbox['height'])/2+$textbox['top'];
				}
				else{
					$y=$this->h-$textbox['height']+$textbox['top'];
				}
			}
			if ($isstrX){
				if ($x=='left'){
					$x=0;
				}
				elseif ($x=='center'){
					$x=($this->w-$textbox['width'])/2;
				}
				else{
					$x=($this->w-$textbox['width']);
				}
			}
		}
		imagettftext($this->getImage(),$this->font_size,$angle,$x,$y,$this->getColor($color),$this->font_path,$text);
	}


	#############################################
	#                                           #
	#  ▒▒▒▒  ▒▒  ▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒  ▒▒  ▒▒ ▒▒▒▒▒▒ #
	# ▒▒  ▒▒ ▒▒  ▒▒ ▒ ▒▒ ▒ ▒▒  ▒▒ ▒▒  ▒▒ ▒ ▒▒ ▒ #
	# ▒▒  ▒▒ ▒▒  ▒▒   ▒▒   ▒▒▒▒▒  ▒▒  ▒▒   ▒▒   #
	# ▒▒  ▒▒ ▒▒  ▒▒   ▒▒   ▒▒     ▒▒  ▒▒   ▒▒   #
	#  ▒▒▒▒   ▒▒▒▒   ▒▒▒▒  ▒▒      ▒▒▒▒   ▒▒▒▒  #
	#                                           #
	#############################################
	# render
	# render the current image and put it in ->rendered
	# if path is specified, saves the picture & put the path in ->rendered instead of picture data
	public function render($save_path=null){
		if (!$this->format||stripos('jpg,jpeg,gif,png,webp',$this->format)===false){
			trigger_error('Incorrect image format: must be jpg,jpeg,gif,png,webp',E_USER_WARNING);
			return false;
		}
		if ($this->image){
			if (!$save_path){ob_start();}
			$function='image'.str_replace('jpg','jpeg',$this->format);
			$function($this->image,$save_path,$this->quality);
			if (!$save_path){
				$this->rendered = ob_get_contents();
				ob_end_clean();
			}else{
				$this->rendered = $save_path;
			}
		}else{
			trigger_error('There is no image to render',E_USER_WARNING);
			return false;
		}
	}

	# alias for render("file")
	public function save($save_path){
		$this->render($save_path);
	}

	# displays the current image
	# renders it if needed
	# loads it if a file has been previously saved
	public function display(){
		header ("Content-type: image/".$this->format);
		if (!$this->rendered){
			$this->render();
		}elseif(is_file($this->rendered)){
			exit(file_get_contents($this->rendered));
		}
		exit($this->rendered);
	}


	####################################################
	#                                                  #
	#  ▒▒▒▒  ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒   ▒▒▒▒  #
	# ▒▒   ▒ ▒▒     ▒ ▒▒ ▒ ▒ ▒▒ ▒ ▒▒     ▒▒  ▒▒ ▒▒   ▒ #
	#   ▒▒   ▒▒▒▒▒    ▒▒     ▒▒   ▒▒▒▒▒  ▒▒▒▒▒    ▒▒   #
	# ▒   ▒▒ ▒▒       ▒▒     ▒▒   ▒▒     ▒▒  ▒▒ ▒   ▒▒ #
	#  ▒▒▒▒  ▒▒▒▒▒▒  ▒▒▒▒   ▒▒▒▒  ▒▒▒▒▒▒ ▒▒  ▒▒  ▒▒▒▒  #
	#                                                  #
	####################################################
	public function setImage($value){		
		$this->w=imagesx($value);
		$this->h=imagesy($value);
		$this->image=$value;
	}
	public function setFormat($value){
		if (!is_string($value)){
			trigger_error('Value must be string',E_USER_WARNING);
			return;
		}
		$this->format=strtolower($value);
	}
	public function setQuality($value){
		if (!is_integer($value)){
			trigger_error('Value must be integer',E_USER_WARNING);
			return;
		}
		$this->quality=$value;
	}
	public function setW($value){
		if (!is_integer($value)){
			trigger_error('Value must be integer',E_USER_WARNING);
			return;
		}
		$this->w=$value;
	}
	public function setH($value){
		if (!is_integer($value)){
			trigger_error('Value must be integer',E_USER_WARNING);
			return;
		}
		$this->h=$value;
	}
	public function setColors($value){
		if (!is_array($value)){
			trigger_error('Value must be array',E_USER_WARNING);
			return;
		}
		$this->colors=$value;
	}
	
	public function setFont($path,$size){
		$this->setFontPath($path);
		$this->setFontSize($size);
	}
	
	public function setFontPath($value){
		if (!is_string($value)){
			trigger_error('Value must be string',E_USER_WARNING);
			return;
		}
		$this->font_path=$value;
	}
	public function setFontSize($value){
		if (!is_integer($value)){
			trigger_error('Value must be integer',E_USER_WARNING);
			return;
		}
		$this->font_size=$value;
	}

	####################################################
	#                                                  #
	#  ▒▒▒▒  ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒▒ ▒▒▒▒▒   ▒▒▒▒  #
	# ▒▒     ▒▒     ▒ ▒▒ ▒ ▒ ▒▒ ▒ ▒▒     ▒▒  ▒▒ ▒▒   ▒ #
	# ▒▒ ▒▒▒ ▒▒▒▒▒    ▒▒     ▒▒   ▒▒▒▒▒  ▒▒▒▒▒    ▒▒   #
	# ▒▒  ▒▒ ▒▒       ▒▒     ▒▒   ▒▒     ▒▒  ▒▒ ▒   ▒▒ #
	#  ▒▒▒▒  ▒▒▒▒▒▒  ▒▒▒▒   ▒▒▒▒  ▒▒▒▒▒▒ ▒▒  ▒▒  ▒▒▒▒  #
	#                                                  #
	####################################################

	public function getImage(){
		if (!$this->image){
			trigger_error('There is no image');
		}
		return $this->image;
	}
	public function getFormat(){return $this->format;}
	public function getQuality(){return $this->quality;}
	public function getW(){return $this->w;}
	public function getH(){return $this->h;}
	public function getSize(){return [$this->h,$this->h,'width'=>$this->h,'height'=>$this->h];}
	public function getColors(){return $this->colors;}
	public function getColor($color_or_name=''){
		if (!empty($this->colors[$color_or_name])){
			return $this->colors[$color_or_name];
		}
		return $this->newColor('temp',$color_or_name);
	}
	public function getTransparentColor(){return $this->transparent_color;}
	public function getFontPath(){return $this->font_path;}
	public function getFontSize(){return $this->font_size;}
	//public function getFontColor(){return $this->font_color;}

}

?>